@@ -189,7 +189,7 @@ PrgBody* Preprocessor::addBodyVar(Var bodyId) {
 		prg_->setConflict();
 		return body;
 	}
-	if ((!body->hasHeads() && body->value() != value_false) || !body->relevant()) {
+	if (superfluous(body)) {
 		body->markRemoved();
 		return body;
 	}
@@ -208,7 +208,7 @@ PrgBody* Preprocessor::addBodyVar(Var bodyId) {
 				Var dualAtom = getRootAtom(body->literal());
 				aEq = dualAtom != varMax ? prg_->getAtom(dualAtom) : 0;
 			}
-			if (aEq && aEq->supps_begin()->isBody()) {
+			if (aEq && aEq->supports() && aEq->supps_begin()->isBody()) {
 				rId = aEq->supps_begin()->node();
 				r   = prg_->getBody(rId);	
 				if (r && r->var() == aEq->var()) {
@@ -325,7 +325,7 @@ bool Preprocessor::propagateAtomVar(Var atomId, PrgAtom* a, PrgEdge source) {
 		Var bodyId  = it->var();
 		PrgBody* bn = prg_->getBody(bodyId);
 		if (bn->relevant()) {
-			bool wasSup = bn->isSupported();
+			bool wasSup = bn->isSupported();	
 			bool isSup  = wasSup || (value != value_false && !it->sign() && bn->propagateSupported(atomId));
 			bool seen   = false;
 			bool dirty  = removeAtom || (removeNeg && it->sign());
@@ -381,6 +381,22 @@ bool Preprocessor::hasRootLiteral(PrgBody* body) const {
 		&& getRootAtom(body->literal()) == varMax
 		&& getRootAtom(~body->literal())== varMax;
 }
+// Pre: body is simplified!
+bool Preprocessor::superfluous(PrgBody* body) const {
+	if (!body->relevant()) { return true; }
+	if (!body->hasHeads()) {
+		if (body->value() == value_free) { return true; }
+		if (body->bound() <= 0)          { return true; }
+		if (body->size() == 1)           { 
+			// unit constraint
+			ValueRep exp = body->value() ^ (int)body->goal(0).sign();
+			ValueRep got = prg_->getAtom(body->goal(0).var())->value();
+			assert(got != value_free || !prg_->options().backprop);
+			return got != value_free && (got&value_true) == (exp&value_true);
+		}
+	}
+	return false;
+}
 
 // Simplify the classified body with the given id.
 // Return:
@@ -407,7 +423,7 @@ ValueRep Preprocessor::simplifyBody(PrgBody* b, bool reclass, VarVec& supported)
 				b->clearLiteral(true);
 			}
 		}
-		else if (!b->hasHeads() && b->value() != value_false && b->var() != 0) {
+		else if (b->var() != 0 && superfluous(b)) {
 			// Body is no longer needed. All heads are either superfluous or equivalent
 			// to other atoms. 
 			// Reclassify only if var is not used
