@@ -72,7 +72,7 @@ bool ClaspConfig::addPost(Solver& s) const {
 	return ok && BasicSatConfig::addPost(s);
 }
 /////////////////////////////////////////////////////////////////////////////////////////
-// ClaspFacade::SolveImpl/SolveStrategy/AsyncResult
+// ClaspFacade::SolveData/SolveStrategy/AsyncResult
 /////////////////////////////////////////////////////////////////////////////////////////
 struct ClaspFacade::SolveStrategy {
 public:
@@ -83,13 +83,14 @@ public:
 	bool running() const    { return (state & state_running) != 0; }
 	bool interrupt(int sig) {
 		if (!running()) { return false; }
-		if (!signal || sig < signal) { signal = sig; } 
+		if (!signal || sig < signal) { signal = sig; }
 		return cancel(sig);
 	}
 	void solve(ClaspFacade& f, SolveAlgorithm* algo, EventHandler* h) {
-		this->algo    = algo;
+		this->algo = algo;
 		this->handler = h;
-		state = signal= 0;
+		this->state = 0;
+		this->signal = f.result().signal;
 		doSolve(f);
 	}
 	virtual void release()  {}
@@ -99,69 +100,84 @@ public:
 	SolveAlgorithm*    algo;
 	EventHandler*      handler;
 protected:
-	void solveImpl(ClaspFacade& f, State end);
+	void runAlgo(ClaspFacade& f, State end);
 	virtual void doSolve(ClaspFacade& f) = 0;
 };
-struct ClaspFacade::SolveImpl {
+struct ClaspFacade::SolveData {
 	typedef SingleOwnerPtr<SolveAlgorithm> AlgoPtr;
 	typedef SingleOwnerPtr<Enumerator>     EnumPtr;
-	SolveImpl() : en(0), algo(0), active(0) { }
-	~SolveImpl() { reset(); }
+	SolveData() : en(0), algo(0), active(0), prepared(false), interruptible(false) { }
+	~SolveData() { reset(); }
 	void init(SolveAlgorithm* algo, Enumerator* en) {
-		this->en   = en;
+		this->en = en;
 		this->algo = algo;
 		this->algo->setEnumerator(*en);
+		if (interruptible) {
+			this->algo->enableInterrupts();
+		}
 	}
 	void reset() {
 		if (active)    { interrupt(SolveStrategy::SIGCANCEL); active->release(); active = 0; }
 		if (algo.get()){ algo->resetSolve(); }
 		if (en.get())  { en->reset(); }
+		prepared = false;
 	}
-	void prepare(SharedContext& ctx, SharedMinimizeData* min, int numM) {
+	void prepare(SharedContext& ctx, SharedMinimizeData* min, int numM, EnumMode mode) {
 		CLASP_FAIL_IF(active, "Solve operation still active");
-		int limit = en->init(ctx, min, numM);
-		algo->setEnumLimit(limit ? static_cast<uint64>(limit) : UINT64_MAX);
+		if (ctx.ok() && !ctx.frozen() && !prepared) {
+			if (min) { min = min->share(); }
+			int limit = en->init(ctx, min, numM);
+			algo->setEnumLimit(limit ? static_cast<uint64>(limit) : UINT64_MAX);
+			if (mode == enum_static) { ctx.addUnary(ctx.stepLiteral()); }
+			prepared = true;
+		}
 	}
 	bool update(const Solver& s, const Model& m) { return !active->handler || active->handler->onModel(s, m); }
 	bool interrupt(int sig)                      { return active && active->interrupt(sig); }
-	bool                      solving()   const  { return active && active->running();   }
+	bool                      solving()   const  { return active && active->running(); }
 	const Model*              lastModel() const  { return en.get() ? &en->lastModel() : 0; }
-	const SharedMinimizeData* minimizer() const  { return en.get() ? en->minimizer() : 0; } 
+	const SharedMinimizeData* minimizer() const  { return en.get() ? en->minimizer() : 0; }
 	Enumerator*               enumerator()const  { return en.get(); }
 	int                       modelType() const  { return en.get() ? en->modelType() : 0; }
 	EnumPtr        en;
 	AlgoPtr        algo;
 	SolveStrategy* active;
+	bool           prepared;
+	bool           interruptible;
 };
-void ClaspFacade::SolveStrategy::solveImpl(ClaspFacade& f, State done) {
-	if (state != state_running){ state = state_running; }
-	if (f.result().flags)      { state = done; signal = f.result().signal; return; }
+void ClaspFacade::SolveStrategy::runAlgo(ClaspFacade& f, State done) {
 	struct OnExit {
 		OnExit(SolveStrategy* s, ClaspFacade* x, int st) : self(s), facade(x), endState(st), more(true) {}
-		~OnExit() { 
+		~OnExit() {
 			facade->stopStep(self->signal, !more);
 			if (self->handler) { self->handler->onEvent(StepReady(facade->summary())); }
-			self->state  = endState;
+			self->state = endState;
 		}
 		SolveStrategy* self;
 		ClaspFacade*   facade;
 		int            endState;
 		bool           more;
 	} scope(this, &f, done);
-	f.step_.solveTime = f.step_.unsatTime = RealTime::getTime();
-	scope.more = algo->solve(f.ctx, f.assume_, &f);
+	if (state != state_running){ state = state_running; }
+	if (!signal && f.ctx.ok()) {
+		f.step_.solveTime = f.step_.unsatTime = RealTime::getTime();
+		scope.more = algo->solve(f.ctx, f.assume_, &f);
+	}
+	else {
+		f.ctx.report(message<Event::verbosity_low>(Event::subsystem_solve, "Solving"));
+		scope.more = f.ctx.ok();
+	}
 }
 
 #if WITH_THREADS
-struct ClaspFacade::AsyncSolve : public SolveStrategy, public EventHandler {
+struct ClaspFacade::AsyncSolve : public SolveStrategy, public EventHandler{
 	static EventHandler* asyncModelHandler() { return reinterpret_cast<EventHandler*>(0x1); }
 	AsyncSolve() { refs = 1; }
 	virtual void doSolve(ClaspFacade& f) {
 		if (handler == AsyncSolve::asyncModelHandler()) { handler = this; }
 		algo->enableInterrupts();
-		state  = state_running;
+		state = state_running;
 		result = f.result();
-		if (result.flags) { signal = result.signal; state = state_done; return; }
 		// start solve in worker thread
 		Clasp::thread(std::mem_fun(&AsyncSolve::threadMain), this, &f).swap(task);
 	}
@@ -169,20 +185,20 @@ struct ClaspFacade::AsyncSolve : public SolveStrategy, public EventHandler {
 		return SolveStrategy::cancel(sig) && (sig != SIGCANCEL || wait());
 	}
 	void threadMain(ClaspFacade* f) {
-		try         { solveImpl(*f, state_running); }
+		try         { runAlgo(*f, state_running); }
 		catch (...) { result.flags |= Result::EXT_ERROR; }
 		{
 			unique_lock<Clasp::mutex> lock(this->mqMutex);
 			result = f->result();
-			state  = state_done;
+			state = state_done;
 		}
 		mqCond.notify_one();
 	}
 	virtual void release() {
-		if      (--refs == 1) { interrupt(SIGCANCEL); }
-		else if (refs   == 0) { join(); delete this; }
+		if (--refs == 1) { interrupt(SIGCANCEL); }
+		else if (refs == 0) { join(); delete this; }
 	}
-	bool hasResult()const { return (state & state_result)  != 0; }
+	bool hasResult()const { return (state & state_result) != 0; }
 	bool next() {
 		if (state != state_model) { return false; }
 		unique_lock<Clasp::mutex> lock(mqMutex);
@@ -193,10 +209,10 @@ struct ClaspFacade::AsyncSolve : public SolveStrategy, public EventHandler {
 	bool wait(double s = -1.0) {
 		if (state == state_start) { return false; }
 		if (signal != 0)          { next(); }
-		for (unique_lock<Clasp::mutex> lock(mqMutex); !hasResult(); ) {
+		for (unique_lock<Clasp::mutex> lock(mqMutex); !hasResult();) {
 			if (s < 0.0) { mqCond.wait(lock); }
 			else {
-				try { 
+				try {
 					mqCond.wait_for(lock, tbb::tick_count::interval_t(s));
 					if (!hasResult()) { return false; }
 				}
@@ -208,26 +224,31 @@ struct ClaspFacade::AsyncSolve : public SolveStrategy, public EventHandler {
 				}
 			}
 		}
-		if (state == state_done) { join(); }
+		if (state == state_done && join()) {
+			// Just in case other AsyncSolve objects are waiting on the computation.
+			// This should not happen but there is some existing code that uses
+			// an AsyncSolve object in a separate thread for cancellation.
+			mqCond.notify_all();
+		}
 		return true;
 	}
 	bool onModel(const Solver&, const Model&) {
-		const Result sat = {Result::SAT, 0};
+		const Result sat = { Result::SAT, 0 };
 		unique_lock<Clasp::mutex> lock(mqMutex);
 		state = state_model;
-		result= sat;
+		result = sat;
 		mqCond.notify_one();
 		while (state == state_model && signal == 0) { mqCond.wait(lock); }
 		return signal == 0;
 	}
-	void join() { if (task.joinable()) { task.join(); } }
+	bool join() { if (task.joinable()) { task.join(); return true; } return false; }
 	Clasp::thread             task;   // async solving thread
 	Clasp::mutex              mqMutex;// protects mqCond
 	Clasp::condition_variable mqCond; // for iterating over models one by one
 	Clasp::atomic<int>        refs;   // 1 + #AsyncResult objects
 	Result                    result; // result of async operation
 };
-ClaspFacade::AsyncResult::AsyncResult(SolveImpl& x) : state_(static_cast<AsyncSolve*>(x.active)) { ++state_->refs; }
+ClaspFacade::AsyncResult::AsyncResult(SolveData& x) : state_(static_cast<AsyncSolve*>(x.active)) { ++state_->refs; }
 ClaspFacade::AsyncResult::~AsyncResult() { state_->release(); }
 ClaspFacade::AsyncResult::AsyncResult(const AsyncResult& x) : state_(x.state_) { ++state_->refs; }
 int  ClaspFacade::AsyncResult::interrupted()        const { return state_->signal; }
@@ -244,12 +265,12 @@ ClaspFacade::Result ClaspFacade::AsyncResult::get() const {
 	if (!state_->result.error()) { return state_->result; }
 	throw std::runtime_error("Async operation failed!");
 }
-bool ClaspFacade::AsyncResult::end() const { 
+bool ClaspFacade::AsyncResult::end() const {
 	// first running() handles case where iterator is outdated (state was reset to start)
 	// second running() is used to distinguish models from final result (summary)
 	return !running() || !get().sat() || !running();
 }
-const Model& ClaspFacade::AsyncResult::model() const { 
+const Model& ClaspFacade::AsyncResult::model() const {
 	CLASP_FAIL_IF(state_->state != AsyncSolve::state_model, "Invalid iterator access!");
 	return const_cast<const SolveAlgorithm*>(state_->algo)->enumerator()->lastModel();
 }
@@ -257,14 +278,14 @@ const Model& ClaspFacade::AsyncResult::model() const {
 /////////////////////////////////////////////////////////////////////////////////////////
 // ClaspFacade
 /////////////////////////////////////////////////////////////////////////////////////////
-ClaspFacade::ClaspFacade() : config_(0) {}
+ClaspFacade::ClaspFacade() : config_(0) { step_.init(*this); }
 ClaspFacade::~ClaspFacade() { }
 void ClaspFacade::discardProblem() {
-	config_  = 0;
+	config_ = 0;
 	builder_ = 0;
 	lpStats_ = 0;
-	solve_   = 0;
-	accu_    = 0;
+	solve_ = 0;
+	accu_ = 0;
 	step_.init(*this);
 	if (ctx.numConstraints() || ctx.numVars()) { ctx.reset(); }
 }
@@ -276,7 +297,7 @@ void ClaspFacade::init(ClaspConfig& config, bool discard) {
 		ctx.report(warning(Event::subsystem_facade, "Reasoning mode requires domain heuristic and is ignored!"));
 		config_->solve.enumMode = EnumOptions::enum_auto;
 	}
-	SolveImpl::EnumPtr e(config.solve.createEnumerator(config.solve));
+	SolveData::EnumPtr e(config.solve.createEnumerator(config.solve));
 	if (e.get() == 0) { e = EnumOptions::nullEnumerator(); }
 	if (config.solve.numSolver() > 1 && !e->supportsParallel()) {
 		ctx.report(warning(Event::subsystem_facade, "Selected reasoning mode implies #Threads=1."));
@@ -288,8 +309,8 @@ void ClaspFacade::init(ClaspConfig& config, bool discard) {
 		p->setOptions(config.asp);
 		p->setNonHcfConfiguration(config.testerConfig());
 	}
-	if (!solve_.get()) { solve_ = new SolveImpl(); }
-	SolveImpl::AlgoPtr a(config.solve.createSolveObject());
+	if (!solve_.get()) { solve_ = new SolveData(); }
+	SolveData::AlgoPtr a(config.solve.createSolveObject());
 	solve_->init(a.release(), e.release());
 	if (discard) { startStep(0); }
 }
@@ -301,8 +322,8 @@ void ClaspFacade::initBuilder(ProgramBuilder* in) {
 }
 
 ProgramBuilder& ClaspFacade::start(ClaspConfig& config, ProblemType t) {
-	if      (t == Problem_t::SAT) { return startSat(config); }
-	else if (t == Problem_t::PB)  { return startPB(config);  }
+	if (t == Problem_t::SAT) { return startSat(config); }
+	else if (t == Problem_t::PB)  { return startPB(config); }
 	else if (t == Problem_t::ASP) { return startAsp(config); }
 	else                          { throw std::domain_error("Unknown problem type!"); }
 }
@@ -322,8 +343,8 @@ PBBuilder& ClaspFacade::startPB(ClaspConfig& config) {
 Asp::LogicProgram& ClaspFacade::startAsp(ClaspConfig& config, bool enableUpdates) {
 	init(config, true);
 	Asp::LogicProgram* p = new Asp::LogicProgram();
-	lpStats_             = new Asp::LpStats;
-	p->accu              = lpStats_.get();
+	lpStats_ = new Asp::LpStats;
+	p->accu = lpStats_.get();
 	initBuilder(p);
 	p->setOptions(config.asp);
 	p->setNonHcfConfiguration(config.testerConfig());
@@ -337,7 +358,10 @@ bool ClaspFacade::enableProgramUpdates() {
 	if (!accu_.get()) {
 		builder_->updateProgram();
 		ctx.requestStepVar();
-		solve_->algo->enableInterrupts();
+		if (!solve_->interruptible) {
+			solve_->interruptible = true;
+			solve_->algo->enableInterrupts();
+		}
 		accu_ = new Summary();
 		accu_->init(*this);
 		accu_->step = UINT32_MAX;
@@ -346,78 +370,95 @@ bool ClaspFacade::enableProgramUpdates() {
 }
 
 ProgramBuilder& ClaspFacade::update(bool reloadConfig) {
-	CLASP_ASSERT_CONTRACT(builder_.get() && !solving());
+	CLASP_ASSERT_CONTRACT(config_ && program() && !solving());
 	CLASP_ASSERT_CONTRACT_MSG(step_.result.signal != SIGINT, "Interrupt not handled!");
-	solve_->reset();
-	if (reloadConfig) { init(*config_, false); }
-	if (builder_->frozen()) {
-		startStep(step()+1);
-		if (builder_->updateProgram()) { assume_.clear(); }
-		else                           { stopStep(0, true); }
+	if (reloadConfig) { 
+		init(*config_, false); 
 	}
-	return *builder_;
+	if (solved()) {
+		startStep(step() + 1);
+	}
+	if (builder_->frozen()) {
+		assume_.clear();
+		builder_->updateProgram();
+	}
+	if (ctx.frozen()) {
+		ctx.unfreeze();
+	}
+	step_.result.signal = 0;
+	solve_->reset();
+	return *program();
 }
+
 bool ClaspFacade::terminate(int signal) {
 	if (solve_.get() && solve_->interrupt(signal)) {
 		return true;
 	}
 	// solving not active or not interruptible
-	stopStep(signal, false);
+	if (!solved() && !step_.result.signal) {
+		step_.result.signal = static_cast<uint8>(signal);
+	}
 	return false;
 }
 
 bool ClaspFacade::prepare(EnumMode enumMode) {
-	CLASP_ASSERT_CONTRACT(config_ && !solving());
-	bool ok = this->ok();
+	CLASP_ASSERT_CONTRACT(solve_.get() && !solving() && !solved());
 	SharedMinimizeData* m = 0;
-	EnumOptions& en       = config_->solve;
-	if (builder_.get() && (ok = builder_->endProgram()) == true) {
-		builder_->getAssumptions(assume_);
-		if ((m  = en.optMode != MinimizeMode_t::ignore ? builder_->getMinimizeConstraint(&en.optBound) : 0) != 0) {
-			ok = m->setMode(en.optMode, en.optBound);
+	ProgramBuilder*   prg = program();
+	EnumOptions& en = config_->solve;
+	if (prepared()) { return ok(); }
+	if (prg && prg->endProgram()) {
+		assume_.clear();
+		prg->getAssumptions(assume_);
+		if ((m = en.optMode != MinimizeMode_t::ignore ? prg->getMinimizeConstraint(&en.optBound) : 0) != 0) {
+			if (!m->setMode(en.optMode, en.optBound)) {
+				assume_.push_back(~ctx.stepLiteral());
+			}
 			if (en.optMode == MinimizeMode_t::enumerate && en.optBound.empty()) {
 				ctx.report(warning(Event::subsystem_facade, "opt-mode=enum: no bound given, optimize statement ignored."));
 			}
 		}
 	}
-	if (ok && (!ctx.frozen() || (ok = ctx.unfreeze()) == true)) {
-		// Step literal only needed for volatile enumeration
-		ok = (enumMode == enum_volatile || ctx.addUnary(ctx.stepLiteral()));
-		if (ok) { solve_->prepare(ctx, m ? m->share() : 0, en.numModels); }
-	}
+	CLASP_ASSERT_CONTRACT(!ctx.ok() || !ctx.frozen());
+	solve_->prepare(ctx, m, en.numModels, enumMode);
 	if      (!accu_.get())  { builder_ = 0; }
 	else if (lpStats_.get()){ static_cast<Asp::LogicProgram*>(builder_.get())->dispose(false); }
-	return (ok && ctx.endInit()) || (stopStep(0, true), false);
+	return ctx.ok() && ctx.endInit();
 }
+
 void ClaspFacade::assume(Literal p) {
 	assume_.push_back(p);
 }
 void ClaspFacade::assume(const LitVec& ext) {
 	assume_.insert(assume_.end(), ext.begin(), ext.end());
 }
-bool ClaspFacade::solving() const { return solve_.get() && solve_->solving(); }
+bool ClaspFacade::prepared()    const { return solve_.get() && solve_->prepared; }
+bool ClaspFacade::solving()     const { return solve_.get() && solve_->solving(); }
+bool ClaspFacade::solved()      const { return step_.totalTime >= 0; }
+bool ClaspFacade::interrupted() const { return result().interrupted(); }
 
 const ClaspFacade::Summary& ClaspFacade::shutdown() {
-	if      (!config_) { step_.init(*this); }
-	else if (solving()){ terminate(SIGINT); }
-	else               { stopStep(0, false); }
+	if (solve_.get()) {
+		solve_->interrupt(SolveStrategy::SIGCANCEL);
+		stopStep(step_.result.signal, !ok());
+	}
 	return accu_.get() && accu_->step ? *accu_ : step_;
 }
 
 ClaspFacade::Result ClaspFacade::solve(EventHandler* handler) {
-	CLASP_ASSERT_CONTRACT(!solving());
+	prepare();
 	struct SyncSolve : public SolveStrategy {
-		SyncSolve(SolveImpl& s) : x(&s) { x->active = this; }
+		SyncSolve(SolveData& s) : x(&s) { x->active = this; }
 		~SyncSolve()                    { x->active = 0;    }
-		virtual void doSolve(ClaspFacade& f) { solveImpl(f, state_done); }
-		SolveImpl* x;
+		virtual void doSolve(ClaspFacade& f) { runAlgo(f, state_done); }
+		SolveData* x;
 	} syncSolve(*solve_);
 	syncSolve.solve(*this, solve_->algo.get(), handler);
 	return result();
 }
 #if WITH_THREADS
 ClaspFacade::AsyncResult ClaspFacade::solveAsync(EventHandler* handler) {
-	CLASP_ASSERT_CONTRACT(!solving());
+	prepare();
 	solve_->active = new AsyncSolve();
 	solve_->active->solve(*this, solve_->algo.get(), handler);
 	return AsyncResult(*solve_);
@@ -435,21 +476,21 @@ void ClaspFacade::startStep(uint32 n) {
 	ctx.report(StepStart(*this));
 }
 ClaspFacade::Result ClaspFacade::stopStep(int signal, bool complete) {
-	if (step_.totalTime < 0) {
+	if (!solved()) {
 		double t = RealTime::getTime();
 		step_.totalTime += t;
-		step_.cpuTime   += ProcessTime::getTime();
+		step_.cpuTime += ProcessTime::getTime();
 		if (step_.solveTime) {
 			step_.solveTime = t - step_.solveTime;
 			step_.unsatTime = complete ? t - step_.unsatTime : 0;
 		}
-		Result res = {uint8(0), uint8(signal)};
+		Result res = { uint8(0), uint8(signal) };
 		if (complete) { res.flags = uint8(step_.enumerated() ? Result::SAT : Result::UNSAT) | Result::EXT_EXHAUST; }
 		else          { res.flags = uint8(step_.enumerated() ? Result::SAT : Result::UNKNOWN); }
-		if (signal)   { res.flags|= uint8(Result::EXT_INTERRUPT); }
+		if (signal)   { res.flags |= uint8(Result::EXT_INTERRUPT); }
 		step_.result = res;
 		accuStep();
-		ctx.report(StepReady(step_)); 
+		ctx.report(StepReady(step_));
 	}
 	return result();
 }
@@ -457,14 +498,14 @@ void ClaspFacade::accuStep() {
 	if (accu_.get() && accu_->step != step_.step){
 		if (step_.stats()) { ctx.accuStats(); }
 		accu_->totalTime += step_.totalTime;
-		accu_->cpuTime   += step_.cpuTime;
+		accu_->cpuTime += step_.cpuTime;
 		accu_->solveTime += step_.solveTime;
 		accu_->unsatTime += step_.unsatTime;
-		accu_->numEnum   += step_.numEnum;
+		accu_->numEnum += step_.numEnum;
 		// no aggregation
 		if (step_.numEnum) { accu_->satTime = step_.satTime; }
-		accu_->step       = step_.step;
-		accu_->result     = step_.result;
+		accu_->step = step_.step;
+		accu_->result = step_.result;
 	}
 }
 bool ClaspFacade::onModel(const Solver& s, const Model& m) {
@@ -484,11 +525,11 @@ ExpectedQuantity ClaspFacade::getStatImpl(const char* path, bool keys) const {
 #define GET_KEYS(o, path) ( ExpectedQuantity((o).keys(path)) )
 #define GET_OBJ(o, path)  ( ExpectedQuantity((o)[path]) )
 #define COMMON_KEYS "ctx.\0solvers.\0solver.\0costs.\0time_total\0time_cpu\0time_solve\0time_unsat\0time_sat\0enumerated\0optimal\0step\0result\0"
-	static const char* _keys[] = {"accu.\0hccs.\0hcc.\0lp.\0" COMMON_KEYS, 0, "accu.\0lp.\0" COMMON_KEYS, "accu.\0" COMMON_KEYS };
+	static const char* _keys[] = { "accu.\0hccs.\0hcc.\0lp.\0" COMMON_KEYS, 0, "accu.\0lp.\0" COMMON_KEYS, "accu.\0" COMMON_KEYS };
 	enum ObjId { id_hccs, id_hcc, id_lp, id_ctx, id_solvers, id_solver, id_costs, id_total, id_cpu, id_solve, id_unsat, id_sat, id_num, id_opt, id_step, id_result };
 	if (!path)  path = "";
 	std::size_t kLen = 0;
-	int         oId  = lpStats_.get() ? ((ctx.sccGraph.get() && ctx.sccGraph->numNonHcfs() != 0) ? id_hccs : id_lp) : id_ctx;
+	int         oId = lpStats_.get() ? ((ctx.sccGraph.get() && ctx.sccGraph->numNonHcfs() != 0) ? id_hccs : id_lp) : id_ctx;
 	const char* keyL = _keys[oId];
 	bool        accu = matchStatPath(path, "accu");
 	if (!*path) { 
@@ -497,7 +538,7 @@ ExpectedQuantity ClaspFacade::getStatImpl(const char* path, bool keys) const {
 	}
 	accu = accu && step() != 0;
 	for (const char* k = keyL + 6; (kLen = std::strlen(k)) != 0; k += kLen + 1, ++oId) {
-		bool match = k[kLen-1] == '.' ? matchStatPath(path, k, kLen-1) : std::strcmp(path, k) == 0;
+		bool match = k[kLen - 1] == '.' ? matchStatPath(path, k, kLen - 1) : std::strcmp(path, k) == 0;
 		if (match) {
 			if (!*path && !keys){ return ExpectedQuantity::error_ambiguous_quantity; }
 			switch(oId) {
@@ -559,7 +600,7 @@ ExpectedQuantity ClaspFacade::getStatImpl(const char* path, bool keys) const {
 			}
 		}
 	}
-	return ExpectedQuantity::error_unknown_quantity; 
+	return ExpectedQuantity::error_unknown_quantity;
 #undef GET_KEYS
 #undef GET_OBJ
 #undef COMMON_KEYS
@@ -569,7 +610,7 @@ ExpectedQuantity ClaspFacade::getStat(const char* path)const {
 	return config_ && step_.totalTime >= 0.0 ? getStatImpl(path, false) : ExpectedQuantity::error_not_available;
 }
 const char*      ClaspFacade::getKeys(const char* path)const {
-	ExpectedQuantity x =  config_ && step_.totalTime >= 0.0 ? getStatImpl(path, true) : ExpectedQuantity::error_not_available;
+	ExpectedQuantity x = config_ && step_.totalTime >= 0.0 ? getStatImpl(path, true) : ExpectedQuantity::error_not_available;
 	if (x.valid()) { return (const char*)static_cast<uintp>(x.rep); }
 	return x.error() == ExpectedQuantity::error_unknown_quantity ? 0 : "\0";
 }
@@ -581,13 +622,13 @@ ExpectedQuantity ClaspFacade::getStat(const SharedContext& ctx, const char* key,
 		if (!x.valid()) { return x; }
 		res.rep += x.rep;
 	}
-	return res;	
+	return res;
 }
 Enumerator* ClaspFacade::enumerator() const { return solve_.get() ? solve_->enumerator() : 0; }
 /////////////////////////////////////////////////////////////////////////////////////////
 // ClaspFacade::Summary
 /////////////////////////////////////////////////////////////////////////////////////////
-void ClaspFacade::Summary::init(ClaspFacade& f)  { std::memset(this, 0, sizeof(Summary)); facade = &f;}
+void ClaspFacade::Summary::init(ClaspFacade& f)  { std::memset(this, 0, sizeof(Summary)); facade = &f; }
 int ClaspFacade::Summary::stats()          const { return ctx().master()->stats.level(); }
 const Model* ClaspFacade::Summary::model() const { return facade->solve_.get() ? facade->solve_->lastModel() : 0; }
 bool ClaspFacade::Summary::optimize()      const {
@@ -598,11 +639,11 @@ bool ClaspFacade::Summary::optimize()      const {
 }
 const char* ClaspFacade::Summary::consequences() const {
 	int mt = facade->solve_.get() ? facade->solve_->modelType() : 0;
-	if ( (mt & CBConsequences::brave_consequences)    == CBConsequences::brave_consequences)   { return "Brave"; }
-	if ( (mt & CBConsequences::cautious_consequences) == CBConsequences::cautious_consequences){ return "Cautious"; }
+	if ((mt & CBConsequences::brave_consequences) == CBConsequences::brave_consequences)   { return "Brave"; }
+	if ((mt & CBConsequences::cautious_consequences) == CBConsequences::cautious_consequences){ return "Cautious"; }
 	return 0;
 }
-uint64 ClaspFacade::Summary::optimal() const { 
+uint64 ClaspFacade::Summary::optimal() const {
 	const Model* m = model();
 	if (m && m->opt) {
 		return !m->consequences() ? std::max(m->num, uint64(1)) : uint64(complete());
