@@ -441,6 +441,9 @@ void JsonOutput::printKeyValue(const char* k, uint64 v) {
 }
 void JsonOutput::printKeyValue(const char* k, uint32 v) { return printKeyValue(k, uint64(v)); }
 void JsonOutput::printKeyValue(const char* k, double v) {
+#if defined(__cplusplus) && __cplusplus >= 201103L
+	using std::isnan;
+#endif
 	if (!isnan(v)) { printf("%s%-*s\"%s\": %.3f", open_, indent(), " ", k, v); } 
 	else           { printf("%s%-*s\"%s\": %s", open_, indent(), " ", k, "null"); }
 	open_ = ",\n";
