@@ -635,7 +635,9 @@ const std::size_t domLen_s = std::strlen(domSym_s);
 bool DomEntry::isDomEntry(const SymbolType& sym) { return !sym.name.empty() && std::strncmp(sym.name.c_str(), domSym_s, domLen_s) == 0; }
 bool DomEntry::isHeadOf(const char* head, const SymbolType& domSym) {
 	assert(isDomEntry(domSym));
-	return std::strncmp(head, domSym.name.c_str() + domLen_s, std::strlen(head)) == 0; 
+	std::size_t len = std::strlen(head);
+	const char* dom = domSym.name.c_str() + domLen_s;
+	return std::strncmp(head, dom, len) == 0 && dom[len] == ',';
 }
 bool DomEntry::Cmp::operator()(const SymbolType& lhs, const SymbolType& rhs) const {
 	assert(DomEntry::isDomEntry(lhs) && DomEntry::isDomEntry(rhs));
@@ -673,12 +675,12 @@ void DomEntry::init(Literal lit, const SymbolType& domSym) {
 	char* next;
 	this->value = Range<int>(INT16_MIN, INT16_MAX).clamp(std::strtol(rule, &next, 10));
 	CLASP_FAIL_IF(rule == next || *(rule = next) == 0, "Invalid value in heuristic predicate!");
+	this->prio = value > 0 ? value : -value;
 	if (mod == mod_sign) {
 		pref  = value_free + ValueRep(value != 0) + ValueRep(value < 0);
 		if (lit.sign() && pref) { pref ^= 3u; }
 		value = pref;
 	}
-	prio = value > 0 ? value : -value;
 	// extract optional priority
 	if (*rule == ',') { // _heuristic/4
 		prio = Range<int>(0, INT16_MAX).clamp(std::strtol(++rule, &next, 10));
@@ -731,7 +733,7 @@ void DomainHeuristic::initScores(Solver& s, bool moms) {
 	std::sort(domTab.begin(), domTab.end(), DomEntry::Cmp());
 	// apply modifications to relevant atoms
 	Literal dyn;
-	uint32 nPrio  = (uint32)prios_.size(), cPrio = nPrio, inc = uint32(s.symbolTable().incremental());
+	uint32 nKey   = (uint32)prios_.size(), uKey = nKey;
 	uint32 nRules = (uint32)domTab.size();
 	if (&s == s.sharedContext()->master()) { domMin_ = &s.sharedContext()->symbolTable().domLits; }
 	for (SymbolTable::const_iterator sIt = s.symbolTable().begin(), end = s.symbolTable().end(); sIt != end && nRules; ++sIt) {
@@ -744,33 +746,36 @@ void DomainHeuristic::initScores(Solver& s, bool moms) {
 			e.init(sIt->second.lit, *rIt);
 			if (e.mod == DomEntry::mod_init && !s.isTrue(e.cond))   { continue; }
 			ev = e.lit.var();
-			if (score_[ev].domKey >= nPrio){
-				score_[ev].setDom(nPrio++);
+			if (score_[ev].domKey >= nKey){
+				score_[ev].setDom(nKey++);
 				prios_.push_back(DomPrio());
 				prios_.back().clear();
-				if (inc) { cPrio = nPrio; }
 			}
 			int mods = 1;
 			if (e.mod == DomEntry::mod_tf) { e.mod = DomEntry::mod_level; ++mods; }
 			for (;;) {
 				if (e.prio >= prio(ev, e.mod)){
 					if      (s.isTrue(e.cond)) { prio(ev, e.mod) = e.prio; }
-					else if (e.cond != dyn)    { s.addWatch(e.cond, this, (uint32)actions_.size()); dyn = e.cond; inc = true; }
+					else if (e.cond != dyn)    { s.addWatch(dyn = e.cond, this, (uint32)actions_.size()); }
 					else                       { actions_.back().comp = 1; }
-					if (addAction(s, e, init)) { cPrio = nPrio; }
+					if (addAction(s, e, init) && score_[ev].domKey >= uKey) {
+						uKey = score_[ev].domKey + 1;
+					}
 				}
 				if (--mods == 0) { break; }
 				e.mod   = DomEntry::mod_sign;
 				e.value = (int16)e.pref;
 			}
 		}
-		if (ev) {
-			if (init)          { score_[ev].value += init;  }
-			if (nPrio != cPrio){ --nPrio; prios_.pop_back();}
+		if (ev && init) {
+			score_[ev].value += init;  
 		}
-		assert(nPrio == cPrio);
 	}
 	DomTable().swap(domTab);
+	if (s.symbolTable().incremental()) { uKey = nKey; }
+	if (uKey != nKey) {
+		PrioVec(prios_.begin(), prios_.begin()+uKey).swap(prios_);
+	}
 	// apply default modification
 	if (defMod_) {
 		MinimizeConstraint* m = s.enumerationConstraint() ? static_cast<EnumerationConstraint*>(s.enumerationConstraint())->minimizer() : 0;
