@@ -194,6 +194,7 @@ bool LparseParser::doParse() {
 	return parseRules()
 		&& parseSymbolTable()
 		&& parseComputeStatement()
+		&& parseExtStatement()
 		&& parseModels()
 		&& endParse();
 }
@@ -280,6 +281,16 @@ bool LparseParser::parseComputeStatement() {
 	return true;
 }
 
+bool LparseParser::parseExtStatement() {
+	input()->skipWhite();
+	if (match(*input(), 'E', false)) {
+		for (int id; input()->skipWhite() && input()->parseInt(id) && id != 0;) {
+			builder_->freeze(id, value_free);
+		}
+	}
+	return true;
+}
+
 bool LparseParser::parseModels() {
 	int m = 1;
 	check(input()->skipWhite() && input()->parseInt(m, 0, INT_MAX), "Number of models expected!");
