@@ -30,10 +30,25 @@ using namespace Clasp::mt;
 
 class FacadeTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST_SUITE(FacadeTest);
+	CPPUNIT_TEST(testPrepareIsIdempotent);
+	CPPUNIT_TEST(testPrepareIsImplicit);
+	CPPUNIT_TEST(testCannotPrepareSolvedProgram);
+	CPPUNIT_TEST(testCannotSolveSolvedProgram);
+	
+	CPPUNIT_TEST(testUpdateWithoutPrepareDoesNotIncStep);
+	CPPUNIT_TEST(testUpdateWithoutSolveDoesNotIncStep);
+	CPPUNIT_TEST(testInterruptBeforePrepareInterruptsNext);
+	CPPUNIT_TEST(testInterruptBeforeSolveInterruptsNext);
+	CPPUNIT_TEST(testShutdownStopsStep);
+
+	CPPUNIT_TEST(testSolveUnderAssumptions);
+	CPPUNIT_TEST(testPrepareTooStrongBound);
+
 	CPPUNIT_TEST(testIncrementalSolve);
 	CPPUNIT_TEST(testIncrementalEnum);
 	CPPUNIT_TEST(testIncrementalCons);
 	CPPUNIT_TEST(testIncrementalMin);
+	CPPUNIT_TEST(testIncrementalMinIgnore);
 	CPPUNIT_TEST(testUpdateConfig);
 	CPPUNIT_TEST(testIncrementalProjectUpdate);
 	CPPUNIT_TEST(testIncrementalConfigUpdateBug);
@@ -57,6 +72,199 @@ class FacadeTest : public CppUnit::TestFixture {
 	CPPUNIT_TEST_SUITE_END(); 
 public:
 	typedef ClaspFacade::Result Result;
+	void addProgram(Clasp::Asp::LogicProgram& prg) {
+		prg.setAtomName(1, "a").setAtomName(2, "b");
+		prg.startRule().addHead(1).addToBody(2, false).endRule();
+		prg.startRule().addHead(2).addToBody(1, false).endRule();
+	}
+	
+	void testPrepareIsIdempotent() {
+		Clasp::ClaspFacade libclasp;
+		Clasp::ClaspConfig config;
+		config.solve.numModels = 0;
+		Clasp::Asp::LogicProgram& asp = libclasp.startAsp(config, true);
+		addProgram(asp);
+		libclasp.prepare();
+		libclasp.prepare();
+		
+		CPPUNIT_ASSERT(libclasp.solve().sat());
+		CPPUNIT_ASSERT(libclasp.summary().numEnum == 2);
+		CPPUNIT_ASSERT(libclasp.summary().step == 0);
+	}
+
+	void testPrepareIsImplicit() {
+		Clasp::ClaspFacade libclasp;
+		Clasp::ClaspConfig config;
+		config.solve.numModels = 0;
+		Clasp::Asp::LogicProgram& asp = libclasp.startAsp(config);
+		addProgram(asp);
+
+		CPPUNIT_ASSERT(libclasp.solve().sat());
+		CPPUNIT_ASSERT(libclasp.summary().numEnum == 2);
+		CPPUNIT_ASSERT(libclasp.summary().step == 0);
+	}
+
+	void testCannotPrepareSolvedProgram() {
+		Clasp::ClaspFacade libclasp;
+		Clasp::ClaspConfig config;
+		config.solve.numModels = 0;
+		Clasp::Asp::LogicProgram& asp = libclasp.startAsp(config, true);
+		addProgram(asp);
+		libclasp.prepare();
+		CPPUNIT_ASSERT(libclasp.solve().sat());
+		CPPUNIT_ASSERT_THROW(libclasp.prepare(), std::logic_error);
+	}
+	void testCannotSolveSolvedProgram() {
+		Clasp::ClaspFacade libclasp;
+		Clasp::ClaspConfig config;
+		config.solve.numModels = 0;
+		Clasp::Asp::LogicProgram& asp = libclasp.startAsp(config, true);
+		addProgram(asp);
+		libclasp.prepare();
+		CPPUNIT_ASSERT(libclasp.solve().sat());
+		CPPUNIT_ASSERT_THROW(libclasp.solve(), std::logic_error);
+	}
+	void testUpdateWithoutPrepareDoesNotIncStep() {
+		Clasp::ClaspFacade libclasp;
+		Clasp::ClaspConfig config;
+		config.solve.numModels = 0;
+		Clasp::Asp::LogicProgram& asp = libclasp.startAsp(config, true);
+		addProgram(asp);
+		CPPUNIT_ASSERT(libclasp.update().ok());
+		CPPUNIT_ASSERT(libclasp.update().ok());
+		libclasp.prepare();
+		CPPUNIT_ASSERT(libclasp.solve().sat());
+		CPPUNIT_ASSERT(libclasp.summary().numEnum == 2);
+		CPPUNIT_ASSERT(libclasp.summary().step == 0);
+	}
+	void testUpdateWithoutSolveDoesNotIncStep() {
+		Clasp::ClaspFacade libclasp;
+		Clasp::ClaspConfig config;
+		config.solve.numModels = 0;
+		Clasp::Asp::LogicProgram& asp = libclasp.startAsp(config, true);
+		addProgram(asp);
+		libclasp.prepare();
+		CPPUNIT_ASSERT(libclasp.update().ok());
+		libclasp.prepare();
+		
+		CPPUNIT_ASSERT(libclasp.solve().sat());
+		CPPUNIT_ASSERT(libclasp.summary().numEnum == 2);
+		CPPUNIT_ASSERT(libclasp.summary().step == 0);
+	}
+
+	void testInterruptBeforePrepareInterruptsNext() {
+		Clasp::ClaspFacade libclasp;
+		Clasp::ClaspConfig config;
+		config.solve.numModels = 0;
+		Clasp::Asp::LogicProgram& asp = libclasp.startAsp(config, true);
+		addProgram(asp);
+		struct FH : EventHandler {
+			FH() : finished(0) {}
+			virtual void onEvent(const Event& ev) {
+				finished += event_cast<ClaspFacade::StepReady>(ev) != 0;
+			}
+			int finished;
+		} fh;
+		CPPUNIT_ASSERT(libclasp.terminate(1) == false);
+		libclasp.prepare();
+		CPPUNIT_ASSERT(libclasp.solve(&fh).interrupted());
+		CPPUNIT_ASSERT(libclasp.solved());
+		CPPUNIT_ASSERT(fh.finished == 1);
+	}
+
+	void testInterruptBeforeSolveInterruptsNext() {
+		Clasp::ClaspFacade libclasp;
+		Clasp::ClaspConfig config;
+		config.solve.numModels = 0;
+		Clasp::Asp::LogicProgram& asp = libclasp.startAsp(config, true);
+		addProgram(asp);
+		struct FH : EventHandler {
+			FH() : finished(0) {}
+			virtual void onEvent(const Event& ev) {
+				finished += event_cast<ClaspFacade::StepReady>(ev) != 0;
+			}
+			int finished;
+		} fh;
+		libclasp.prepare();
+		CPPUNIT_ASSERT(libclasp.terminate(1) == false);
+		CPPUNIT_ASSERT(!libclasp.solved());
+		CPPUNIT_ASSERT(libclasp.solve(&fh).interrupted());
+		CPPUNIT_ASSERT(libclasp.solved());
+		CPPUNIT_ASSERT(fh.finished == 1);
+	}
+
+
+	void testShutdownStopsStep() {
+		Clasp::ClaspFacade libclasp;
+		Clasp::ClaspConfig config;
+		config.solve.numModels = 0;
+		Clasp::Asp::LogicProgram& asp = libclasp.startAsp(config, true);
+		addProgram(asp);
+		libclasp.prepare();
+		libclasp.shutdown();
+		CPPUNIT_ASSERT(libclasp.solved());
+	}
+
+	void testSolveUnderAssumptions() {
+		Clasp::ClaspFacade libclasp;
+		Clasp::ClaspConfig config;
+		config.solve.numModels = 0;
+		Clasp::Asp::LogicProgram& asp = libclasp.startAsp(config, true);
+		addProgram(asp);
+		Var ext = asp.newAtom();
+		asp.setAtomName(ext, "ext");
+		asp.freeze(ext, value_true);
+		libclasp.prepare();
+		LitVec assume(1, asp.getLiteral(1));
+		struct MH : public Clasp::EventHandler {
+			MH() : models(0) {}
+			bool onModel(const Clasp::Solver&, const Clasp::Model& m) {
+				for (LitVec::const_iterator it = exp.begin(), end = exp.end(); it != end; ++it) {
+					CPPUNIT_ASSERT(m.isTrue(*it));
+				}
+				++models;
+				return true;
+			}
+			LitVec exp;
+			int    models;
+		} mh1, mh2;
+		mh1.exp.push_back(asp.getLiteral(1));
+		mh1.exp.push_back(~asp.getLiteral(2));
+		mh1.exp.push_back(asp.getLiteral(ext));
+		libclasp.assume(assume);
+		libclasp.solve(&mh1);
+		CPPUNIT_ASSERT(mh1.models == 1);
+		libclasp.update();
+		asp.freeze(ext, value_false);
+		assume.assign(1, asp.getLiteral(2));
+		mh2.exp.push_back(~asp.getLiteral(1));
+		mh2.exp.push_back(asp.getLiteral(2));
+		mh2.exp.push_back(~asp.getLiteral(ext));
+		libclasp.prepare();
+		libclasp.assume(assume);
+		libclasp.solve(&mh2);
+		CPPUNIT_ASSERT(mh2.models == 1);
+		libclasp.update();
+		libclasp.prepare();
+		libclasp.solve();
+		CPPUNIT_ASSERT(libclasp.summary().numEnum == 2);
+	}
+	
+	void testPrepareTooStrongBound() {
+		Clasp::ClaspFacade libclasp;
+		Clasp::ClaspConfig config;
+		config.solve.numModels = 0;
+		config.solve.optBound.assign(1, 0);
+		Clasp::Asp::LogicProgram& asp = libclasp.startAsp(config, true);
+		addProgram(asp);
+		Var c = asp.newAtom();
+		asp.setAtomName(c, "c");
+		asp.startRule().addHead(c).endRule();
+		asp.startRule(Asp::OPTIMIZERULE).addToBody(c, true).addToBody(1, true).addToBody(2, true).endRule();
+		libclasp.prepare();
+		CPPUNIT_ASSERT(libclasp.solve().unsat());
+	}
+
 	void testRunIncremental(Result::Base stop, int maxStep, int minStep, Result::Base expectedRes, int expectedSteps) {
 		ClaspConfig config;
 		ClaspFacade f;
@@ -94,7 +302,7 @@ public:
 				asp.setAtomName(8,"f").startRule(Asp::CHOICERULE).addHead(8).endRule();
 				break;
 			}
-			if (!f.prepare()) { break; }
+			f.prepare();
 			res = f.solve();
 		} while (--maxStep && ((minStep > 0 && --minStep) || res != stop));
 		CPPUNIT_ASSERT_EQUAL(expectedRes,  (Result::Base)f.result());
@@ -114,7 +322,7 @@ public:
 		Clasp::Asp::LogicProgram& asp = libclasp.startAsp(config, true);
 		asp.setAtomName(1, "a");
 		asp.startRule(Clasp::Asp::CHOICERULE).addHead(1).endRule();
-		libclasp.prepare(Clasp::ClaspFacade::enum_volatile);
+		libclasp.prepare();
 		CPPUNIT_ASSERT(libclasp.solve().sat());
 		CPPUNIT_ASSERT(libclasp.summary().numEnum == 2);
 		CPPUNIT_ASSERT(libclasp.update().ok());
@@ -157,6 +365,24 @@ public:
 		CPPUNIT_ASSERT(libclasp.solve().sat());
 		CPPUNIT_ASSERT(libclasp.summary().numEnum == 8);
 	}
+	void testIncrementalMinIgnore() {
+		Clasp::ClaspFacade libclasp;
+		Clasp::ClaspConfig config;
+		config.solve.optMode = MinimizeMode_t::ignore;
+		config.solve.numModels = 0;
+		Clasp::Asp::LogicProgram& asp = libclasp.startAsp(config, true);
+		asp.setAtomName(1, "a").setAtomName(2, "b");
+		asp.startRule(Clasp::Asp::CHOICERULE).addHead(1).addHead(2).endRule();
+		asp.startRule(Clasp::Asp::OPTIMIZERULE).addToBody(1, true).addToBody(2, true).endRule();
+		libclasp.prepare();
+		CPPUNIT_ASSERT(libclasp.solve().sat());
+		CPPUNIT_ASSERT(libclasp.summary().numEnum == 4u);
+		config.solve.optMode = MinimizeMode_t::optimize;
+		libclasp.update(true);
+		libclasp.prepare();
+		CPPUNIT_ASSERT(libclasp.solve().sat());
+		CPPUNIT_ASSERT(libclasp.summary().numEnum == 1u);
+	}
 	void testUpdateConfig() {
 		Clasp::ClaspFacade libclasp;
 		Clasp::ClaspConfig config;
@@ -315,6 +541,7 @@ public:
 		libclasp.prepare();
 		CPPUNIT_ASSERT(libclasp.solve().sat() && libclasp.summary().numEnum == 10);
 	}
+
 	void testStats() {
 		Clasp::ClaspFacade libclasp;
 		Clasp::ClaspConfig config;
@@ -337,6 +564,7 @@ public:
 		CPPUNIT_ASSERT(libclasp.getStat("costs.0") == double(libclasp.summary().costs()->at(0)));
 		CPPUNIT_ASSERT(libclasp.getStat("optimal") == double(libclasp.summary().optimal()));
 	}
+
 	void getKeys(const ClaspFacade& f, std::vector<std::string>& out, const std::string& p) {
 		typedef std::pair<std::string, double> Stat;
 		std::size_t len = 0;
@@ -372,7 +600,7 @@ public:
 		Asp::LogicProgram& asp = libclasp.startAsp(config, true);
 		asp.setAtomName(1, "a");
 		asp.startRule().addHead(1).addToBody(1, false).endRule();
-		libclasp.prepare(ClaspFacade::enum_volatile);
+		libclasp.prepare();
 		AsyncResult it = libclasp.startSolveAsync();
 		CPPUNIT_ASSERT(it.end());
 		CPPUNIT_ASSERT(it.get().unsat());
@@ -384,7 +612,7 @@ public:
 		Asp::LogicProgram& asp = libclasp.startAsp(config, true);
 		asp.setAtomName(1, "a");
 		asp.startRule(Asp::CHOICERULE).addHead(1).endRule();
-		libclasp.prepare(ClaspFacade::enum_volatile);
+		libclasp.prepare();
 		libclasp.terminate(2);
 		AsyncResult it = libclasp.startSolveAsync();
 		CPPUNIT_ASSERT(it.end());
@@ -397,7 +625,7 @@ public:
 		Asp::LogicProgram& asp = libclasp.startAsp(config, true);
 		asp.setAtomName(1, "a");
 		asp.startRule(Asp::CHOICERULE).addHead(1).endRule();
-		libclasp.prepare(ClaspFacade::enum_volatile);
+		libclasp.prepare();
 		AsyncResult it = libclasp.startSolveAsync();
 		while (!it.end()) { 
 			CPPUNIT_ASSERT(it.model().num == 1);
@@ -405,7 +633,7 @@ public:
 		}
 		CPPUNIT_ASSERT(it.end() && !libclasp.solving() && it.interrupted());
 		libclasp.update();
-		libclasp.prepare(ClaspFacade::enum_volatile);
+		libclasp.prepare();
 		it = libclasp.startSolveAsync();
 		int mod = 0;
 		while (!it.end()) { ++mod; it.next(); }
@@ -417,7 +645,7 @@ public:
 		Asp::LogicProgram& asp = libclasp.startAsp(config, true);
 		asp.setAtomName(1, "a");
 		asp.startRule(Asp::CHOICERULE).addHead(1).endRule();
-		libclasp.prepare(ClaspFacade::enum_volatile);
+		libclasp.prepare();
 		{ AsyncResult it = libclasp.startSolveAsync(); }
 		CPPUNIT_ASSERT(!libclasp.solving() && libclasp.result().interrupted());
 	}
@@ -428,7 +656,7 @@ public:
 		Asp::LogicProgram& asp= libclasp->startAsp(config, true);
 		asp.setAtomName(1, "a");
 		asp.startRule(Asp::CHOICERULE).addHead(1).endRule();
-		libclasp->prepare(ClaspFacade::enum_volatile);
+		libclasp->prepare();
 		handle = new AsyncResult( libclasp->startSolveAsync() );
 		delete libclasp;
 		CPPUNIT_ASSERT(handle->interrupted());
@@ -441,11 +669,11 @@ public:
 		Asp::LogicProgram& asp = libclasp.startAsp(config, true);
 		asp.setAtomName(1, "a");
 		asp.startRule(Asp::CHOICERULE).addHead(1).endRule();
-		libclasp.prepare(ClaspFacade::enum_volatile);
+		libclasp.prepare();
 		handle = new AsyncResult( libclasp.solveAsync() );
 		handle->wait();
 		libclasp.update();
-		libclasp.prepare(ClaspFacade::enum_volatile);
+		libclasp.prepare();
 		AsyncResult* it = new AsyncResult(libclasp.startSolveAsync());
 		delete handle;
 		CPPUNIT_ASSERT(!it->interrupted() && libclasp.solving());
@@ -459,11 +687,11 @@ public:
 		Asp::LogicProgram& asp = libclasp.startAsp(config, true);
 		asp.setAtomName(1, "a");
 		asp.startRule(Asp::CHOICERULE).addHead(1).endRule();
-		libclasp.prepare(ClaspFacade::enum_volatile);
+		libclasp.prepare();
 		AsyncResult step0 = libclasp.solveAsync();
 		step0.wait();
 		libclasp.update();
-		libclasp.prepare(ClaspFacade::enum_volatile);
+		libclasp.prepare();
 		AsyncResult step1 = libclasp.startSolveAsync();
 		
 		CPPUNIT_ASSERT_EQUAL(false, step0.cancel());
